@@ -1,6 +1,6 @@
 from PyQt6.QtCore import Qt, QObject, QEvent, QSize
-from PyQt6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit, QComboBox, QDialog, QToolButton, QWidgetAction, QCheckBox, QTableWidget, QHeaderView
-from PyQt6.QtGui import QIcon
+from PyQt6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit, QComboBox, QDialog, QToolButton, QWidgetAction, QCheckBox, QTableWidget, QHeaderView, QApplication, QStyle, QStyledItemDelegate
+from PyQt6.QtGui import QIcon, QTextDocument
 import re
 import requests
 import importlib.resources
@@ -9,8 +9,18 @@ main_dir = importlib.resources.files("gui_utilities")
 icons_dir = main_dir / "icons"
 tlds_path = main_dir / "tlds"
 
-def create_window(title, background_color = "#1e1e1e"):
-    window = QWidget()
+def latex_to_html(expression):
+    html_expression = expression
+    html_expression = re.sub(r"\\frac\{(.*?)\}\{(.*?)\}", r"<span style='font-size:90%;'>\1⁄\2</span>", html_expression)
+    html_expression = re.sub(r"_\{([^}]+)\}", r"<sub>\1</sub>", html_expression)
+    html_expression = re.sub(r"\^\{([^}]+)\}", r"<sup>\1</sup>", html_expression)
+    html_expression = re.sub(r"_([a-zA-Z0-9]+)", r"<sub>\1</sub>", html_expression)
+    html_expression = re.sub(r"\^([a-zA-Z0-9]+)", r"<sup>\1</sup>", html_expression)
+    html_expression = html_expression.replace(" ", "&nbsp;")
+    return html_expression
+
+def create_window(title, background_color = "#1e1e1e", parent = None):
+    window = QWidget(parent)
     window.setObjectName("window")
     window.setWindowTitle(title)
     window.setStyleSheet(f"#window {{background-color: {background_color};}}")
@@ -33,7 +43,9 @@ def create_menu(
     title_padding_bottom = None,
     title_border_width = 0,
     title_border_color = "#ffffff",
-    title_border_radius = 0
+    title_border_radius = 0,
+    math_expression = False,
+    title_alignment = Qt.AlignmentFlag.AlignCenter
 ):
     main_layout = QVBoxLayout()
     main_layout.setContentsMargins(25, 25, 25, 25)
@@ -50,7 +62,9 @@ def create_menu(
         padding_bottom = title_padding_bottom,
         border_width = title_border_width,
         border_color = title_border_color,
-        border_radius = title_border_radius
+        border_radius = title_border_radius,
+        math_expression = math_expression,
+        alignment = title_alignment
     ))
     body_layout = QHBoxLayout()
     main_layout.addLayout(body_layout)
@@ -102,7 +116,9 @@ def create_header(
     title_padding_bottom = None,
     title_border_width = 0,
     title_border_color = "#ffffff",
-    title_border_radius = 0
+    title_border_radius = 0,
+    math_expression = False,
+    title_alignment = Qt.AlignmentFlag.AlignCenter
 ):
     main_layout = QVBoxLayout()
     main_layout.setContentsMargins(margin_left, margin_top, margin_right, margin_bottom)
@@ -119,7 +135,9 @@ def create_header(
         padding_bottom = title_padding_bottom,
         border_width = title_border_width,
         border_color = title_border_color,
-        border_radius = title_border_radius
+        border_radius = title_border_radius,
+        math_expression = math_expression,
+        alignment = title_alignment
     )
     main_layout.addLayout(title_layout)
     main_layout.addStretch()
@@ -138,11 +156,12 @@ def create_title(
     padding_bottom = None,
     border_width = 0,
     border_color = "#ffffff",
-    border_radius = 0
+    border_radius = 0,
+    math_expression = False,
+    alignment = Qt.AlignmentFlag.AlignCenter
 ):
     title_layout = QHBoxLayout()
     title_layout.setContentsMargins(0, 0, 0, 25)
-    title_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
     title_label = create_label(
         text = text,
         font_family = font_family,
@@ -157,7 +176,9 @@ def create_title(
         padding_bottom = padding_bottom,
         border_width = border_width,
         border_color = border_color,
-        border_radius = border_radius
+        border_radius = border_radius,
+        math_expression = math_expression,
+        alignment = alignment
     )
     title_layout.addWidget(title_label)
     return title_layout
@@ -184,9 +205,19 @@ def create_label(
         disabled_background_color = "transparent",
         disabled_border_width = 0,
         disabled_border_color = "#4a4a4a",
+        math_expression = False,
+        transparent_for_mouse = False,
+        alignment = None,
+        word_wrap = False,
         parent = None
 ):
-    label = QLabel(text, parent)
+    if math_expression:
+        label = QLabel(latex_to_html(text), parent)
+        label.setTextFormat(Qt.TextFormat.RichText)
+    else: label = QLabel(text, parent)
+    if transparent_for_mouse: label.setAttribute(Qt.WidgetAttribute.WA_TransparentForMouseEvents, True)
+    if alignment: label.setAlignment(alignment)
+    if word_wrap: label.setWordWrap(True)
     padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
@@ -243,10 +274,12 @@ def create_button(
     disabled_background_color = "#2d2d2d",
     disabled_border_width = 2,
     disabled_border_color = "#4a4a4a",
-    maximum_width = 360
+    maximum_width = 360,
+    math_expression = False,
+    parent = None
 ):
-    button = QPushButton()
-    button.setMaximumWidth(maximum_width)
+    button = QPushButton(parent)
+    if maximum_width is not None: button.setMaximumWidth(maximum_width)
     main_layout = QVBoxLayout()
     button.setLayout(main_layout)
     main_layout.setContentsMargins(0, 0, 0, 0)
@@ -260,9 +293,11 @@ def create_button(
         padding_top = padding_top,
         padding_right = padding_right,
         padding_bottom = padding_bottom,
-        disabled_border_width = disabled_border_width
+        disabled_border_width = disabled_border_width,
+        transparent_for_mouse = True,
+        alignment = Qt.AlignmentFlag.AlignCenter,
+        math_expression = math_expression
     )
-    label.setAlignment(Qt.AlignmentFlag.AlignCenter)
     label.setWordWrap(True)
     main_layout.addWidget(label)
     button.setFixedHeight(main_layout.sizeHint().height() + 2 * border_width)
@@ -316,6 +351,7 @@ def create_text_box(
     border_width = 2,
     border_color = "#5c5c5c",
     border_radius = 0,
+    placeholder_font_color = "#888888",
     hover_background_color = "#333333",
     hover_border_width = 3,
     hover_border_color = "#777777",
@@ -331,9 +367,10 @@ def create_text_box(
     focused_show_text_icon_url = str(icons_dir / "focused_show_text_icon.png"),
     focused_hide_text_icon_url = str(icons_dir / "focused_hide_text_icon.png"),
     hide_text = False,
-    math_expression = False
+    math_expression = False,
+    parent = None
 ):
-    text_box = QLineEdit()
+    text_box = QLineEdit(parent)
     padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
@@ -367,37 +404,6 @@ def create_text_box(
         }}
     """
     text_box.setStyleSheet(style_sheet)
-
-    def latex_to_html(expression):
-        html_expression = expression
-        html_expression = re.sub(r"\\frac\{(.*?)\}\{(.*?)\}", r"<span style='font-size:90%;'>\1⁄\2</span>", html_expression)
-        html_expression = re.sub(r"([a-zA-Z])_(\d+)", r"\1<sub>\2</sub>", html_expression)
-        html_expression = re.sub(r"([a-zA-Z0-9])\^(\d+)", r"\1<sup>\2</sup>", html_expression)
-        html_expression = html_expression.replace(" ", "&nbsp;")
-        return html_expression
-    
-    if not math_expression: text_box.setPlaceholderText(placeholder_text)
-    else:
-        html_placeholder = latex_to_html(placeholder_text)
-        placeholder_label = create_label(
-            text = html_placeholder,
-            font_color = font_color,
-            font_family = font_family,
-            font_size = font_size,
-            background_color = "transparent",
-            hover_background_color = "transparent",
-            disabled_background_color = "transparent",
-            parent = text_box
-        )
-        placeholder_label.setTextFormat(Qt.TextFormat.RichText)
-        placeholder_label.move(padding_left_value + 2, padding_top_value + 2)
-        placeholder_label.show()
-
-        def update_placeholder_visibility():
-            placeholder_label.setVisible(text_box.text() == "")
-        
-        text_box.textChanged.connect(update_placeholder_visibility)
-        update_placeholder_visibility()
     if hide_text:
         show_text_icon = QIcon(show_text_icon_url)
         hide_text_icon = QIcon(hide_text_icon_url)
@@ -411,38 +417,83 @@ def create_text_box(
         toggle_text_visibility_button.setIconSize(QSize(25, 25))
         toggle_text_visibility_action = QWidgetAction(text_box)
         toggle_text_visibility_action.setDefaultWidget(toggle_text_visibility_button)
-        style_sheet = f"""
-            QToolButton {{
+        text_box.addAction(toggle_text_visibility_action, QLineEdit.ActionPosition.TrailingPosition)
+        toggle_text_visibility_button.setStyleSheet("""
+            QToolButton {
                 background-color: transparent;
                 border: none;
                 margin-right: 10px;
-            }}
-        """
-        toggle_text_visibility_button.setStyleSheet(style_sheet)
-        text_box.addAction(toggle_text_visibility_action, QLineEdit.ActionPosition.TrailingPosition)
-
+            }
+        """)
+        
         def update_icon():
             is_password = text_box.echoMode() == QLineEdit.EchoMode.Password
             if text_box.hasFocus(): icon = focused_hide_text_icon if is_password else focused_show_text_icon
             else: icon = hide_text_icon if is_password else show_text_icon
             toggle_text_visibility_button.setIcon(icon)
-
+        
         def toggle_visibility():
             if text_box.echoMode() == QLineEdit.EchoMode.Password: text_box.setEchoMode(QLineEdit.EchoMode.Normal)
             else: text_box.setEchoMode(QLineEdit.EchoMode.Password)
             update_icon()
-
+        
         toggle_text_visibility_button.clicked.connect(toggle_visibility)
 
-        class FocusWatcher(QObject):
-            def eventFilter(self, watched, event):
-                if event.type() in (QEvent.Type.FocusIn, QEvent.Type.FocusOut): update_icon()
-                return super().eventFilter(watched, event)
+        class _IconFocusWatcher(QObject):
 
-        focus_watcher = FocusWatcher(text_box)
-        text_box.installEventFilter(focus_watcher)
-        setattr(text_box, "focus_watcher", focus_watcher)
-        update_icon()
+            def __init__(self, watched, on_focus_change):
+                super().__init__(watched)
+                self._watched = watched
+                self._on_focus_change = on_focus_change
+            
+            def eventFilter(self, watched, event):
+                if watched is self._watched and event.type() in (QEvent.Type.FocusIn, QEvent.Type.FocusOut):
+                    if callable(self._on_focus_change): self._on_focus_change()
+                return super().eventFilter(watched, event)
+        
+        icon_focus_watcher = _IconFocusWatcher(text_box, update_icon)
+        text_box.installEventFilter(icon_focus_watcher)
+        setattr(text_box, "_icon_focus_watcher", icon_focus_watcher)
+    
+    placeholder_label = create_label(
+        text = placeholder_text,
+        font_family = font_family,
+        font_size = font_size,
+        font_color = placeholder_font_color,
+        font_weight = font_weight,
+        padding = padding,
+        padding_left = padding_left,
+        padding_top = padding_top,
+        padding_right = padding_right,
+        padding_bottom = padding_bottom,
+        math_expression = math_expression,
+        transparent_for_mouse = True,
+        parent = text_box
+    )
+    placeholder_label.move(0, 2)
+    
+    def update_placeholder_visibility():
+        has_text = bool(text_box.text().strip())
+        has_focus = text_box.hasFocus()
+        placeholder_label.setVisible(not has_text and not has_focus)
+        if hide_text: update_icon()
+    
+    class _PlaceholderFocusWatcher(QObject):
+
+        def __init__(self, watched, on_focus_change):
+            super().__init__(watched)
+            self._watched = watched
+            self._on_focus_change = on_focus_change
+        
+        def eventFilter(self, watched, event):
+            if watched is self._watched and event.type() in (QEvent.Type.FocusIn, QEvent.Type.FocusOut):
+                if callable(self._on_focus_change): self._on_focus_change()
+            return super().eventFilter(watched, event)
+    
+    placeholder_focus_watcher = _PlaceholderFocusWatcher(text_box, update_placeholder_visibility)
+    text_box.installEventFilter(placeholder_focus_watcher)
+    setattr(text_box, "_placeholder_focus_watcher", placeholder_focus_watcher)
+    update_placeholder_visibility()
     return text_box
 
 def create_combo_box(
@@ -472,21 +523,108 @@ def create_combo_box(
     dropdown_background_color = "#1e1e1e",
     dropdown_selection_background_color = "#0078d7",
     dropdown_border_width = 1,
-    dropdown_border_color = "#5c5c5c"
+    dropdown_border_color = "#5c5c5c",
+    math_expression = False,
+    parent = None
 ):
-    combo_box = QComboBox()
-    combo_box.setPlaceholderText(f"{placeholder_text}")
-    combo_box.addItems(items)
-    combo_box.setCurrentIndex(-1)
-    
+    combo_box = QComboBox(parent)
+    combo_box.setAccessibleName("combo_box")
     padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
     padding_bottom_value = padding_bottom if padding_bottom is not None else padding
+    if math_expression:
+        placeholder_label = create_label(
+            text = placeholder_text,
+            font_family = font_family,
+            font_size = font_size,
+            font_color = placeholder_font_color,
+            padding = padding,
+            padding_left = padding_left,
+            padding_top = padding_top,
+            padding_right = padding_right,
+            padding_bottom = padding_bottom,
+            math_expression = True,
+            transparent_for_mouse = True,
+            parent = combo_box
+        )
+        placeholder_label.move(0, 2)
 
+        def update_placeholder_visibility():
+            has_selection = combo_box.currentIndex() != -1
+            placeholder_label.setVisible(not has_selection)
+
+        combo_box.currentIndexChanged.connect(update_placeholder_visibility)
+        update_placeholder_visibility()
+
+        class RichTextDelegate(QStyledItemDelegate):
+            def __init__(
+                self,
+                parent = None,
+                font_color = "#ffffff",
+                selection_font_color = "#ffffff"
+            ):
+                super().__init__(parent)
+                self.font_color = font_color
+                self.selection_font_color = selection_font_color
+
+            def paint(self, painter, option, index):
+                if option.state & QStyle.StateFlag.State_Selected: text_color = self.selection_font_color
+                else: text_color = self.font_color
+                document = QTextDocument()
+                html = f"<span style = \"color:{text_color};\">{index.data(Qt.ItemDataRole.DisplayRole)}</span>"
+                document.setHtml(html)
+                option.text = ""
+                QApplication.style().drawControl(QStyle.ControlElement.CE_ItemViewItem, option, painter)
+                painter.save()
+                painter.translate(option.rect.left(), option.rect.top() + (option.rect.height() - document.size().height()) / 2)
+                document.drawContents(painter)
+                painter.restore()
+        
+        delegate = RichTextDelegate(
+            parent = combo_box,
+            font_color = dropdown_font_color,
+            selection_font_color = font_color
+        )
+        combo_box.setItemDelegate(delegate)
+        combo_box.clear()
+        for item in items: combo_box.addItem(latex_to_html(item))
+        combo_box.view().setTextElideMode(Qt.TextElideMode.ElideNone)
+    else:
+        combo_box.setPlaceholderText(placeholder_text)
+        combo_box.addItems(items)
+    combo_box.setCurrentIndex(-1)
+    if math_expression:
+        selected_item_label = create_label(
+            text = "",
+            font_family = font_family,
+            font_size = font_size,
+            font_color = font_color,
+            padding = padding,
+            padding_left = padding_left,
+            padding_top = padding_top,
+            padding_right = padding_right,
+            padding_bottom = padding_bottom,
+            math_expression = True,
+            transparent_for_mouse = True,
+            alignment = Qt.AlignmentFlag.AlignCenter,
+            parent = combo_box
+        )
+        selected_item_label.setVisible(False)
+
+        def update_selected_item_display(index):
+            if index != -1:
+                html_text = combo_box.itemText(index)
+                selected_item_label.setText(html_text)
+                selected_item_label.setVisible(True)
+            else:
+                selected_item_label.setVisible(False)
+
+        combo_box.currentIndexChanged.connect(update_selected_item_display)
+    
     def get_stylesheet(font_color):
         return f"""
-            QComboBox {{
+            QComboBox[accessibleName="combo_box"] {{
                 font-family: {font_family};
                 font-size: {font_size}px;
                 color: {font_color};
@@ -498,12 +636,12 @@ def create_combo_box(
                 border: {border_width}px solid {border_color};
                 border-radius: {border_radius}px;
             }}
-            QComboBox:hover {{
+            QComboBox[accessibleName="combo_box"]:hover {{
                 background-color: {hover_background_color};
                 border: {hover_border_width}px solid {hover_border_color};
             }}
-            QComboBox:on {{
-                color: {on_font_color};
+            QComboBox[accessibleName="combo_box"]:on {{
+                color: {"transparent" if math_expression else on_font_color};
                 background-color: {on_background_color};
                 border: {on_border_width}px solid {on_border_color};
             }}
@@ -519,8 +657,12 @@ def create_combo_box(
         """
 
     def change_color(index):
-        if index == -1: combo_box.setStyleSheet(get_stylesheet(placeholder_font_color))
-        else: combo_box.setStyleSheet(get_stylesheet(font_color))
+        if index == -1:
+            text_color = "transparent" if math_expression else placeholder_font_color
+            combo_box.setStyleSheet(get_stylesheet(text_color))
+        else:
+            text_color = "transparent" if math_expression else font_color
+            combo_box.setStyleSheet(get_stylesheet(text_color))
     
     combo_box.currentIndexChanged.connect(change_color)
     change_color(-1)
@@ -553,9 +695,10 @@ def create_checkbox(
     disabled_font_color = "#888888",
     disabled_background_color = "#2d2d2d",
     disabled_border_width = 1,
-    disabled_border_color = "#4a4a4a"
+    disabled_border_color = "#4a4a4a",
+    parent = None
 ):
-    check_box = QCheckBox(text)
+    check_box = QCheckBox(text, parent)
     padding_left_value = padding_left if padding_left is not None else padding
     padding_top_value = padding_top if padding_top is not None else padding
     padding_right_value = padding_right if padding_right is not None else padding
@@ -646,9 +789,10 @@ def create_information_message_box(
     button_disabled_font_color = "#888888",
     button_disabled_background_color = "#2d2d2d",
     button_disabled_border_width = 2,
-    button_disabled_border_color = "#4a4a4a"
+    button_disabled_border_color = "#4a4a4a",
+    parent = None
 ):
-    message_box = QDialog()
+    message_box = QDialog(parent)
     message_box.setObjectName("message_box")
     message_box.setWindowFlags(Qt.WindowType.FramelessWindowHint)
     message_box.setFixedWidth(360)
@@ -677,10 +821,10 @@ def create_information_message_box(
         padding_bottom = label_padding_bottom,
         border_width = label_border_width,
         border_color = label_border_color,
-        border_radius = label_border_radius
+        border_radius = label_border_radius,
+        alignment = Qt.AlignmentFlag.AlignCenter
     )
     main_layout.addWidget(text_label)
-    text_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
     text_label.setWordWrap(True)
     accept_button = create_button(
         text = button_text,
@@ -755,9 +899,10 @@ def create_confirmation_message_box(
     button_disabled_font_color = "#888888",
     button_disabled_background_color = "#2d2d2d",
     button_disabled_border_width = 2,
-    button_disabled_border_color = "#4a4a4a"
+    button_disabled_border_color = "#4a4a4a",
+    parent = None
 ):
-    confirm_message_box = QDialog()
+    confirm_message_box = QDialog(parent)
     confirm_message_box.setObjectName("confirm_message_box")
     confirm_message_box.setWindowFlags(Qt.WindowType.FramelessWindowHint)
     confirm_message_box.setFixedWidth(360)
@@ -786,10 +931,10 @@ def create_confirmation_message_box(
         padding_bottom = label_padding_bottom,
         border_width = label_border_width,
         border_color = label_border_color,
-        border_radius = label_border_radius
+        border_radius = label_border_radius,
+        alignment = Qt.AlignmentFlag.AlignCenter
     )
     main_layout.addWidget(text_label)
-    text_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
     text_label.setWordWrap(True)
     buttons_layout = QHBoxLayout()
     main_layout.addLayout(buttons_layout)
@@ -884,9 +1029,10 @@ def create_list_table(
     hover_scrollbar_handle_border_color = "#777777",
     pressed_scrollbar_handle_background_color = "#4a4a4a",
     pressed_scrollbar_handle_border_width = 3,
-    pressed_scrollbar_handle_border_color = "#0078d7"
+    pressed_scrollbar_handle_border_color = "#0078d7",
+    parent = None
 ):
-    list_table = QTableWidget()
+    list_table = QTableWidget(parent)
     list_table.verticalHeader().setVisible(False)
     list_table.setColumnCount(len(column_headers))
     list_table.setHorizontalHeaderLabels(column_headers)
@@ -1096,6 +1242,15 @@ def validate_integer(integer, suffix = "El", field = "campo"):
     if pattern.match(unformatted_integer): return None
     return f"No ha ingresado {"un" if suffix == "El" else "una"} {field} {"válido" if suffix == "El" else "válida"}."
 
+def validate_float(number_str, suffix = "El", field = "campo"):
+    if not number_str or not number_str.strip():
+        return f"{suffix} {field} no puede dejarse {"vacío" if suffix == "El" else "vacía"}."
+    try:
+        float(number_str.replace(",", "."))
+        return None
+    except ValueError:
+        return f"No ha ingresado {"un" if suffix == "El" else "una"} {field} {"válido" if suffix == "El" else "válida"}."
+
 def validate_id(id_str):
     if not id_str or not id_str.strip(): return "El D.N.I. no puede dejarse vacio."
     pattern = re.compile(r"^(?:\d{8}|(?:\d{1,2}\.\d{3}\.\d{3}))$")
@@ -1154,9 +1309,14 @@ def validate_email(email):
     if email_pattern.match(email): return None
     return "No ha ingresado un correo electrónico válido."
 
-def decimal_format(number):
-    if isinstance(number, float) and number.is_integer(): number = int(number)
-    return f"{number:,}".replace(",", "X").replace(".", ",").replace("X", ".")
+def decimal_format(number, decimal_places = None):
+    if isinstance(number, int):
+        return f"{number:,}".replace(",", "X").replace(".", ",").replace("X", ".")
+    elif isinstance(number, float):
+        if not decimal_places: formatted_str = f"{number:,}"
+        else: formatted_str = f"{number:,.{decimal_places}f}"
+        return formatted_str.replace(",", "X").replace(".", ",").replace("X", ".")
+    return str(number)
 
 def format_id(id_string):
     clean_id = id_string.replace(".", "")
